# Generated by Django 4.1.7 on 2023-03-01 11:35

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("logs", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="requestlog",
            name="content_type",
            field=models.CharField(max_length=34, verbose_name="请求类型"),
        ),
        migrations.AlterField(
            model_name="requestlog",
            name="execution_time",
            field=models.DecimalField(
                decimal_places=8, max_digits=10, null=True, verbose_name="执行时间"
            ),
        ),
        migrations.AlterField(
            model_name="requestlog",
            name="ip",
            field=models.CharField(max_length=16, verbose_name="用户IP"),
        ),
        migrations.AlterField(
            model_name="requestlog",
            name="method",
            field=models.CharField(max_length=8, verbose_name="请求方法"),
        ),
    ]
